@@ -70,12 +70,17 @@ mm_real		*mm_real_new (MMRealFormat format, MMRealSymm symm, const int m, const
 void		mm_real_free (mm_real *mm);
 bool		mm_real_realloc (mm_real *mm, const int nnz);
 
+void		mm_real_sort (mm_real *x);
+
+void		mm_real_memcpy (mm_real *dest, const mm_real *src);
 mm_real		*mm_real_copy (const mm_real *mm);
+mm_dense	*mm_real_copy_sparse_to_dense (const mm_sparse *s);
+mm_sparse	*mm_real_copy_dense_to_sparse (const mm_dense *x, const double threshold);
 void		mm_real_set_all (mm_real *mm, const double val);
 
-mm_dense	*mm_real_sparse_to_dense (const mm_sparse *s);
-mm_sparse	*mm_real_dense_to_sparse (const mm_dense *x, const double threshold);
-mm_real		*mm_real_symmetric_to_general (const mm_real *x);
+bool		mm_real_sparse_to_dense (mm_sparse *s);
+bool		mm_real_dense_to_sparse (mm_dense *d, const double threshold);
+bool		mm_real_symmetric_to_general (mm_real *x);
 
 mm_real		*mm_real_eye (MMRealFormat type, const int n);
 
@@ -90,8 +95,11 @@ double		mm_real_xj_sum (const mm_real *x, const int j);
 double		mm_real_xj_ssq (const mm_real *x, const int j);
 double		mm_real_xj_nrm2 (const mm_real *x, const int j);
 
-void		mm_real_x_dot_y (bool trans, const double alpha, const mm_real *x, const mm_dense *y, const double beta, mm_dense *z);
-double		mm_real_xj_trans_dot_y (const mm_real *x, const int j, const mm_dense *y);
+void		mm_real_x_dot_y (const bool trans, const double alpha, const mm_real *x, const mm_dense *y, const double beta, mm_dense *z);
+
+double		mm_real_xj_trans_dot_yk (const mm_real *x, const int j, const mm_dense *y, const int k);
+mm_dense	*mm_real_xj_trans_dot_y (const mm_real *x, const int j, const mm_dense *y);
+
 void		mm_real_axjpy (const double alpha, const mm_real *x, const int j, mm_dense *y);
 void		mm_real_axjpy_atomic (const double alpha, const mm_real *x, const int j, mm_dense *y);
 
